@@ -11,7 +11,8 @@ from xpander_sdk import (
     ToolCallType,
     ToolCallResult,
     LLMProvider,
-    SourceNodeType
+    SourceNodeType,
+    GraphItem
 )
 from ...generic import get_sub_agent_id_from_oas_by_name
 from .base import SDKAdapter
@@ -519,7 +520,7 @@ class ChainlitAdapter(SDKAdapter):
         return tool_calls
 
     @cl.step(type="tool", name="xpander-ai")
-    async def process_tool_calls(self, tool_calls: Dict[int, ChainlitToolCallWithLLMResponse], on_node_enter: Optional[Callable], on_node_error: Optional[Callable], local_tools: Dict[str, Callable] = None) -> str | None:
+    async def process_tool_calls(self, tool_calls: Dict[int, ChainlitToolCallWithLLMResponse], on_node_enter: Optional[Callable], on_node_error: Optional[Callable], agent_custom_functions: list[GraphItem] = [], local_tools: Dict[str, Callable] = None) -> str | None:
         """
         Process tool calls by formatting their payloads and executing them.
 
@@ -575,6 +576,9 @@ class ChainlitAdapter(SDKAdapter):
             ]
         )
         
+        def get_custom_function_graph_item(tc: ToolCall) -> GraphItem | None:
+            return next((gi for gi in agent_custom_functions if gi.name == tc.name or gi.name.replace(" ","_") == tc.name),None)
+        
         for raw_tool_call in tool_calls.values():
             tool_call = raw_tool_call.tool_call
             is_agent_end_tool = True if tool_call.name == AGENT_END_TOOL else False
@@ -584,7 +588,7 @@ class ChainlitAdapter(SDKAdapter):
                 current_step.name = "xpander-ai"
                 current_step.show_input = False
             
-            graph_item = self.agent.graph.find_node_by_item_id(item_id=tool_call.name)
+            graph_item = self.agent.graph.find_node_by_item_id(item_id=tool_call.name) or self.agent.graph.find_node_by_name(name=tool_call.name) or get_custom_function_graph_item(tc=tool_call)
             tool_name = graph_item.name if graph_item else tool_call.name
             
             
@@ -597,15 +601,16 @@ class ChainlitAdapter(SDKAdapter):
                     tool_name = sub_agent_node.name
             else:
                 # report to parent window
-                graph_item = self.agent.graph.find_node_by_item_id(item_id=tool_call.name)
+                graph_item = self.agent.graph.find_node_by_item_id(item_id=tool_call.name) or self.agent.graph.find_node_by_name(name=tool_call.name) or get_custom_function_graph_item(tc=tool_call)
                 if graph_item and on_node_enter:
                     await on_node_enter(tool_call=tool_call,graph_item=graph_item)
                     
             if is_sub_agent:
                 if is_agent_end_tool:
+                    current_step.default_open = True
                     tool_name = f"{self.agent.name} - Final Thoughts"
                 else:
-                    tool_name = f"{self.agent.name} - {tool_call.name}"
+                    tool_name = f"{self.agent.name} - {graph_item.name}"
                     
             elif is_agent_end_tool:
                 tool_name = f"Crafting final answer"
